/*	GNU General Public Licence
	
	This small C program loads the brugs.so ELF shared library and calls the CLI function. 
	Save it as a .c file and then compile it on Linux using 
	
				gcc -o bugs CBugs.c -ldl 
	
	This code should work on a Windows machine if brugs.so is repaced by brugs.dll
	
	A ascii version of this file is in Developer/CBugs.c
	
	***********************************************
	* SOLO
	* 
	* 
	*		gcc -o BlackBox BlackBox.c -ldl 
	*
	************************************************
*/

/*
  История:

  07.04.2009 Добавлена загрузка инициализационной процедуры
             и вызов ее для передачи параметров argc и argv.
  14.01.2009 Первоначальный выпуск
*/

#include <dlfcn.h>
#include <stdio.h>
#include <string.h>


/*
    Принцип работы начального загрузчика среды BlackBox
    ---------------------------------------------------
    
    Модуль DevElfLinker, разработанный Оминк, умеет делать библиотеки 
    (shared objects в терминах Linux). Особенность таких библиотек в том, 
    что при ее загрузке при помощи функции dlopen, производится вызов 
    инициализационного кода библиотеки (подробнее см. "man dlopen").
    DevElfLinker собирает библиотеку так, что инициализационным кодом 
    для библиотеки является инициализационная секция модуля Kernel.
    
    Таким образом, при загрузке библиотеки libBB.so производится
    запуск среды BlackBox. В процессе загрузки по цепочке вызываются 
    инициализационные секции модулей, причем вызывются в том поряке, 
    в каком производилась сборка. Последним вызывается модуль Init.
    
    В текущей реализации (с процедурой InitSystem) код из секции инициализации
    модуля Init перенесен внутрь процедуры InitSystem, а инициализационная 
    секция оставлена пустой. Сделано это для того, чтобы можно было обеспечить 
    передачу командной строки внутрь системы BlackBox.
    
    Можно добиться получения командной строки с помощью разбора стека в 
    процессе загрузки библиотеки, но для использования кода с процедурой 
    есть ряд причин:
    1) Процесс инициализации библиотеки (*.so) неочевиден и нетривиален.
       Автору просто не хватает знаний в программировании на Ассемблере 
       в Linux.
    2) Код с процедурой значительно более читаемый и совершенно очевиден.
       Как и его дальнейшее его использование в модулях Init и далее.
       Что очень соответствует принципам Оберона.
       
    Вопросы, изменения?
    
    Автор: Дмитрий Соломенников ака Димыч, mailto: dmitrys99@mail.ru
    Форум: http://forum.oberoncore.ru
    
*/

int main (int argc, char **argv)
{
    void * handle;
    void (*InitSystem)(int, int);
 
    handle = dlopen("./libBB.so", RTLD_LAZY);
    if (!handle)
      return 1;
      
    *(void **) (&InitSystem) = dlsym(handle, "InitSystem");
    if (!InitSystem) 
	return 2;
	
    InitSystem(argc, (int)argv);
    
    dlclose(handle);
 
    return 0;
	
}
